@@ -15,21 +15,13 @@
 # specific language governing permissions and limitations
 # under the License.
 
-from typing import TYPE_CHECKING, Iterable, Mapping, Optional, Sequence, Union
+import warnings
+from typing import Iterable, Mapping, Optional, Sequence, Union
 
-from pandas import DataFrame
-from tabulate import tabulate
-
-from airflow.exceptions import AirflowException
-from airflow.models import BaseOperator
-from airflow.providers.slack.hooks.slack_webhook import SlackWebhookHook
-from airflow.providers.snowflake.hooks.snowflake import SnowflakeHook
-
-if TYPE_CHECKING:
-    from airflow.utils.context import Context
+from airflow.providers.slack.transfers.sql_to_slack import SqlToSlackOperator
 
 
-class SnowflakeToSlackOperator(BaseOperator):
+class SnowflakeToSlackOperator(SqlToSlackOperator):
     """
     Executes an SQL statement in Snowflake and sends the results to Slack. The results of the query are
     rendered into the 'slack_message' parameter as a Pandas dataframe using a JINJA variable called '{{
@@ -48,7 +40,7 @@ class SnowflakeToSlackOperator(BaseOperator):
         SQL results
     :param snowflake_conn_id: Reference to
         :ref:`Snowflake connection id<howto/connection:snowflake>`
-    :param slack_conn_id: The connection id for Slack
+    :param slack_conn_id: The connection id for Slack.
     :param results_df_name: The name of the JINJA template's dataframe variable, default is 'results_df'
     :param parameters: The parameters to pass to the SQL query
     :param warehouse: The Snowflake virtual warehouse to use to run the SQL query
@@ -56,7 +48,7 @@ class SnowflakeToSlackOperator(BaseOperator):
     :param schema: The schema to run the SQL against in Snowflake
     :param role: The role to use when connecting to Snowflake
     :param slack_token: The token to use to authenticate to Slack. If this is not provided, the
-        'webhook_token' attribute needs to be specified in the 'Extra' JSON field against the slack_conn_id
+        'webhook_token' attribute needs to be specified in the 'Extra' JSON field against the slack_conn_id.
     """
 
     template_fields: Sequence[str] = ('sql', 'slack_message')
@@ -80,8 +72,6 @@ class SnowflakeToSlackOperator(BaseOperator):
         slack_token: Optional[str] = None,
         **kwargs,
     ) -> None:
-        super().__init__(**kwargs)
-
         self.snowflake_conn_id = snowflake_conn_id
         self.sql = sql
         self.parameters = parameters
@@ -94,62 +84,31 @@ class SnowflakeToSlackOperator(BaseOperator):
         self.slack_message = slack_message
         self.results_df_name = results_df_name
 
-    def _get_query_results(self) -> DataFrame:
-        snowflake_hook = self._get_snowflake_hook()
-
-        self.log.info('Running SQL query: %s', self.sql)
-        df = snowflake_hook.get_pandas_df(self.sql, parameters=self.parameters)
-        return df
-
-    def _render_and_send_slack_message(self, context, df) -> None:
-        # Put the dataframe into the context and render the JINJA template fields
-        context[self.results_df_name] = df
-        self.render_template_fields(context)
-
-        slack_hook = self._get_slack_hook()
-        self.log.info('Sending slack message: %s', self.slack_message)
-        slack_hook.execute()
-
-    def _get_snowflake_hook(self) -> SnowflakeHook:
-        return SnowflakeHook(
-            snowflake_conn_id=self.snowflake_conn_id,
-            warehouse=self.warehouse,
-            database=self.database,
-            role=self.role,
-            schema=self.schema,
+        warnings.warn(
+            """
+            SnowflakeToSlackOperator is deprecated.
+            Please use `airflow.providers.slack.transfers.sql_to_slack.SqlToSlackOperator`.
+            """,
+            DeprecationWarning,
+            stacklevel=2,
         )
 
-    def _get_slack_hook(self) -> SlackWebhookHook:
-        return SlackWebhookHook(
-            http_conn_id=self.slack_conn_id, message=self.slack_message, webhook_token=self.slack_token
+        hook_params = {
+            "schema": self.schema,
+            "role": self.role,
+            "database": self.database,
+            "warehouse": self.warehouse,
+        }
+        cleaned_hook_params = {k: v for k, v in hook_params.items() if v is not None}
+
+        super().__init__(
+            sql=self.sql,
+            sql_conn_id=self.snowflake_conn_id,
+            slack_conn_id=self.slack_conn_id,
+            slack_webhook_token=self.slack_token,
+            slack_message=self.slack_message,
+            results_df_name=self.results_df_name,
+            parameters=self.parameters,
+            sql_hook_params=cleaned_hook_params,
+            **kwargs,
         )
-
-    def render_template_fields(self, context, jinja_env=None) -> None:
-        # If this is the first render of the template fields, exclude slack_message from rendering since
-        # the snowflake results haven't been retrieved yet.
-        if self.times_rendered == 0:
-            fields_to_render: Iterable[str] = filter(lambda x: x != 'slack_message', self.template_fields)
-        else:
-            fields_to_render = self.template_fields
-
-        if not jinja_env:
-            jinja_env = self.get_template_env()
-
-        # Add the tabulate library into the JINJA environment
-        jinja_env.filters['tabulate'] = tabulate
-
-        self._do_render_template_fields(self, fields_to_render, context, jinja_env, set())
-        self.times_rendered += 1
-
-    def execute(self, context: 'Context') -> None:
-        if not isinstance(self.sql, str):
-            raise AirflowException("Expected 'sql' parameter should be a string.")
-        if self.sql is None or self.sql.strip() == "":
-            raise AirflowException("Expected 'sql' parameter is missing.")
-        if self.slack_message is None or self.slack_message.strip() == "":
-            raise AirflowException("Expected 'slack_message' parameter is missing.")
-
-        df = self._get_query_results()
-        self._render_and_send_slack_message(context, df)
-
-        self.log.debug('Finished sending Snowflake data to Slack')
